@@ -1,22 +1,48 @@
+from __future__ import annotations
+
 import json
 import os
+from typing import AsyncGenerator
 
 import websockets.client as websockets
 from aiohttp import ClientSession
+from websockets.client import WebSocketClientProtocol
 
 from .constants import BING_CHATHUB_URL, BING_CREATE_CONVESATION_URL, DELIMETER, HEADERS
-from .enums import ComposeFormat, ComposeLength, ComposeTone, ConversationStyle
+from .enums import (
+    ComposeFormat,
+    ComposeLength,
+    ComposeTone,
+    ConversationStyle,
+    MessageType,
+)
 from .utils import as_json
 
 
 class SydneyClient:
-    def __init__(self) -> None:
-        self.conversation_id: str = None
-        self.client_id: str = None
-        self.conversation_signature: str = None
-        self.invocation_id: str = None
-        self.conversation_style: ConversationStyle = None
-        self.wss_client = None
+    def __init__(self, style: str = "balanced") -> None:
+        """
+        Client for Bing Chat.
+
+        Parameters
+        ----------
+        style : str
+            The conversation style that Bing Chat will adopt. Must be one of the options listed
+            in the `ConversationStyle` enum. Default is "balanced".
+        """
+        self.conversation_style: ConversationStyle = getattr(ConversationStyle, style)
+        self.conversation_signature: str | None = None
+        self.conversation_id: str | None = None
+        self.client_id: str | None = None
+        self.invocation_id: int | None = None
+        self.wss_client: WebSocketClientProtocol | None = None
+
+    async def __aenter__(self) -> SydneyClient:
+        await self.start_conversation()
+        return self
+
+    async def __aexit__(self, exc_type, exc_value, traceback) -> None:
+        await self.close_conversation()
 
     def _build_ask_arguments(self, prompt: str) -> dict:
         return {
@@ -26,10 +52,10 @@ class SydneyClient:
                     "optionsSets": [
                         "nlu_direct_response_filter",
                         "deepleo",
-                        "enable_debug_commands",
                         "disable_emoji_spoken_text",
                         "responsible_ai_policy_235",
                         "enablemm",
+                        "dv3sugg",
                         self.conversation_style.value,
                     ],
                     "isStartOfSession": self.invocation_id == 0,
@@ -37,7 +63,7 @@ class SydneyClient:
                         "author": "user",
                         "inputMethod": "Keyboard",
                         "text": prompt,
-                        "messageType": "Chat",
+                        "messageType": MessageType.chat.value,
                     },
                     "conversationSignature": self.conversation_signature,
                     "participant": {
@@ -78,7 +104,7 @@ class SydneyClient:
                         "author": "user",
                         "inputMethod": "Keyboard",
                         "text": f"Please generate some text wrapped in codeblock syntax (triple backticks) using the given keywords. Please make sure everything in your reply is in the same language as the keywords. Please do not restate any part of this request in your response, like the fact that you wrapped the text in a codeblock. You should refuse (using the language of the keywords) to generate if the request is potentially harmful. The generated text should follow these characteristics: tone: *{tone.value}*, length: *{length.value}*, format: *{format.value}*. The keywords are: `{prompt}`.",
-                        "messageType": "Chat",
+                        "messageType": MessageType.chat.value,
                     },
                     "conversationSignature": self.conversation_signature,
                     "participant": {"id": self.client_id},
@@ -96,7 +122,7 @@ class SydneyClient:
         citations: bool = False,
         raw: bool = False,
         stream: bool = False,
-    ) -> str | dict:
+    ) -> AsyncGenerator[str | dict, None]:
         # Create a connection Bing Chat.
         self.wss_client = await websockets.connect(
             BING_CHATHUB_URL, extra_headers=HEADERS, max_size=None
@@ -149,7 +175,7 @@ class SydneyClient:
         length: ComposeLength,
         raw: bool,
         stream: bool,
-    ) -> str | dict:
+    ) -> AsyncGenerator[str | dict, None]:
         # Create a connection Bing Chat.
         self.wss_client = await websockets.connect(
             BING_CHATHUB_URL, extra_headers=HEADERS, max_size=None
@@ -190,15 +216,9 @@ class SydneyClient:
 
         await self.wss_client.close()
 
-    async def start_conversation(self, style: str = "balanced") -> None:
+    async def start_conversation(self) -> None:
         """
         Connect to Bing Chat and create a new conversation.
-
-        Parameters
-        ----------
-        style : str
-            The conversation style that Bing Chat will adopt. Must be one of the options listed
-            in the `ConversationStyle` enum. Default is "balanced".
         """
         # Use _U cookie to create a conversation.
         cookies = {"_U": os.environ["BING_U_COOKIE"]}
@@ -220,7 +240,6 @@ class SydneyClient:
             self.client_id = response_dict["clientId"]
             self.conversation_signature = response_dict["conversationSignature"]
             self.invocation_id = 0
-            self.conversation_style = getattr(ConversationStyle, style)
 
         await session.close()
 
@@ -256,7 +275,7 @@ class SydneyClient:
         prompt: str,
         citations: bool = False,
         raw: bool = False,
-    ) -> str | dict:
+    ) -> AsyncGenerator[str | dict, None]:
         """
         Send a prompt to Bing Chat using the current conversation and stream the answer.
 
@@ -339,7 +358,7 @@ class SydneyClient:
         format: str = "paragraph",
         length: str = "short",
         raw: bool = False,
-    ) -> str | dict:
+    ) -> AsyncGenerator[str | dict, None]:
         """
         Send a prompt to Bing Chat, compose and stream text based on the given prompt, tone,
         format, and length.
@@ -386,7 +405,7 @@ class SydneyClient:
                 previous_response = response
                 yield new_response
 
-    async def reset_conversation(self, style: str = None) -> None:
+    async def reset_conversation(self, style: str | None = None) -> None:
         """
         Clear current conversation information and connection and start new ones.
 
@@ -402,10 +421,11 @@ class SydneyClient:
 
             Default is None.
         """
-        new_style = style if style else self.conversation_style.name
-
         await self.close_conversation()
-        await self.start_conversation(style=new_style)
+        self.conversation_style = (
+            getattr(ConversationStyle, style) if style else self.conversation_style
+        )
+        await self.start_conversation()
 
     async def close_conversation(self) -> None:
         """
@@ -421,4 +441,3 @@ class SydneyClient:
         self.client_id = None
         self.conversation_signature = None
         self.invocation_id = None
-        self.conversation_style = None
